==> matrix-compiler-2.0.0-beta.2.tgz: package/src/matrix-compile/matrixCompile.ts <==
// @ts-ignore
import Server from '@hyext/matrix/src/Server'
// @ts-ignore
import outputBundle from '@hyext/matrix/src/shared/output/bundle'
// @ts-ignore
import { mergeConfig } from 'metro-config'
import path from 'path'
import getDefaultConfig, { generateTransformerFile } from '../matrix-config/getDefaultConfig'
import saveAssets from './saveAssets'

import logger from '../lib/logger'
import md5 from '../lib/md5'
import { MetroConfig, } from '../matrix-config/configTypes'
import R from 'ramda'


type Platform = 'ios' | 'android'

export interface BundleConfig {
  entryAbsPath: string
  outputDir: string

  /*
   * 资源文件如图片的保存地址
   * 如果为空, 则保存到jsbundle的同级目录
   * 警告: 这个路径需要和 getDefaultConfig 的 imageBaseURL 配合使用
   */
  assetsDir?: string
  getBundleName?: (platform: Platform, entryAbsPath: string) => string
  platforms?: Platform[]
}

let totalCompileTime = 0
export interface Options {
  projectRoot: string
  bundleConfigs: BundleConfig[]
  imageBaseURL: string

  /*
   * 示例: { 'open-sdk': ['@hyext/hyext-rn-sdk', '@hyext/hyliveext-rn-sdk', 'hyext-rn-sdk', 'hyliveext-rn-sdk'] }
   *
   * app 端处理rn业务时, 会执行他们预置的 open-sdk, 再执行业务代码
   * open-sdk 即是预置的 sdk 名称
   */
  SDKAlias: { [aliasName: string]: string[] }

  metroConfig?: MetroConfig
  filterModules?: Set<string>
  extUuid?: string
  cacheVersion?: string
}


export type CompileOutput = {
  entryAbsPath: string
  bundleAbsPath: string
  bundleMd5: string
  platform: Platform
}

export default async function compile({
  projectRoot,
  filterModules,
  imageBaseURL,

  SDKAlias,

  metroConfig = {},
  bundleConfigs,
  extUuid,
  cacheVersion,
}: Options): Promise<CompileOutput[]> {
  const allStart = Date.now()
  const defaultConfig = getDefaultConfig({
    projectRoot,
    imageBaseURL,
    filterModules: filterModules || new Set(),
    SDKAlias,
    mode: 'production',
    extUuid,
    cacheVersion,
  })

  const babelTransformerPath = generateTransformerFile({
    projectRoot: projectRoot,
    babelPlugins: metroConfig?.transformer?.babelPlugins,
    hyextTransformerConfig: metroConfig?.transformer?.hyextTransformerConfig,
  })

  // const finalConfig = mergeConfig(defaultConfig, metroConfig)
  const finalConfig = mergeConfig(defaultConfig, {
    ...metroConfig,
    transformer: {
      babelTransformerPath,
    },
  })

  const server = new Server({
    ...finalConfig,
    extUuid,
  })

  const bundlePlatform = async ({
    entryAbsPath,
    outputDir,
    assetsDir,
    getBundleName,
  }: BundleConfig, platform: Platform): Promise<CompileOutput> => {
    const bundleStart = Date.now()
    const bundleName = getBundleName ? getBundleName(platform, entryAbsPath) : `${path.basename(entryAbsPath, '.js')}.${platform}.jsbundle`
    const bundleAbsPath = path.resolve(outputDir, bundleName)

    const requestOpts = {
      entryFile: entryAbsPath,
      sourceMapUrl: `${path.basename(bundleAbsPath)}.map`,
      dev: false,
      minify: true,
      platform,
    }

    const buildStart = Date.now()
    const bundle = await outputBundle.build(server, requestOpts)
    console.log(`build ${path.basename(entryAbsPath)}, ${platform}: ${Date.now() - buildStart}ms`)


    const outputOpts = {
      bundleOutput: bundleAbsPath,
      bundleEncoding: 'utf8',
      dev: false,
      indexedRamBundle: false,
      platform,
      assetsDest: assetsDir || outputDir,
      sourcemapOutput: `${bundleAbsPath}.map`,
      // sourcemapUseAbsolutePath: false,
      // sourcemapSourcesRoot: '',
    }
    const saveStart = Date.now()
    await outputBundle.save(bundle, outputOpts, logger.info)

    if (platform === 'ios') {
      const outputAssets = await server.getAssets({
        ...Server.DEFAULT_BUNDLE_OPTIONS,
        ...requestOpts,
        bundleType: 'todo',
      })

      // 只需要保存一份资源即可
      await saveAssets(outputAssets, platform, assetsDir || outputDir)
    }
    console.log(`save ${path.basename(entryAbsPath)}, ${platform}: ${Date.now() - saveStart}ms`)

    const bundleMd5 = await md5(bundleAbsPath)

    console.log(`build+save for ${path.basename(entryAbsPath)} ${platform}: ${Date.now() - bundleStart}ms`)

    return {
      entryAbsPath,
      bundleAbsPath,
      bundleMd5,
      platform,
    }
  }


  const promises: Array<Promise<CompileOutput>> = []
  bundleConfigs.forEach(bundleConfig => {
    (bundleConfig.platforms || ['ios', 'android']).forEach(platform => {
      promises.push(bundlePlatform(bundleConfig, platform))
    })
  })

  try {
    const compileOutputs = await Promise.all(promises)
    totalCompileTime += (Date.now() - allStart)
    return compileOutputs
  } finally {
    console.log(`totalCompileTime: ${totalCompileTime}`)
    server.end()
  }
}

==> matrix-compiler-2.0.1-beta.2.tgz: package/src/matrix-compile/matrixCompile.ts <==
// @ts-ignore
import Server from '@hyext/matrix/src/Server'
// @ts-ignore
import outputBundle from '@hyext/matrix/src/shared/output/bundle'
// @ts-ignore
import { mergeConfig } from 'metro-config'
import path from 'path'
import getDefaultConfig, { generateTransformerFile } from '../matrix-config/getDefaultConfig'
// import saveAssets from './saveAssets'
import saveAssets from '@react-native-community/cli-plugin-metro/build/commands/bundle/saveAssets'

import logger, { dbg } from '../lib/logger'
import md5 from '../lib/md5'
import { MetroConfig, } from '../matrix-config/configTypes'
import R from 'ramda'
import axios from 'axios'
import { validateHyUI, validateAnimatedValue, minValidVersion } from '../lib/validateAnimatedValue'


type Platform = 'ios' | 'android'

export interface BundleConfig {
  entryAbsPath: string
  outputDir: string

  /*
   * 资源文件如图片的保存地址
   * 如果为空, 则保存到jsbundle的同级目录
   * 警告: 这个路径需要和 getDefaultConfig 的 imageBaseURL 配合使用
   */
  assetsDir?: string
  getBundleName?: (platform: Platform, entryAbsPath: string) => string
  platforms?: Platform[]
}

export interface Options {
  projectRoot: string
  bundleConfigs: BundleConfig[]
  imageBaseURL: string

  /*
   * 示例: { 'open-sdk': ['@hyext/hyext-rn-sdk', '@hyext/hyliveext-rn-sdk', 'hyext-rn-sdk', 'hyliveext-rn-sdk'] }
   *
   * app 端处理rn业务时, 会执行他们预置的 open-sdk, 再执行业务代码
   * open-sdk 即是预置的 sdk 名称
   */
  SDKAlias: { [aliasName: string]: string[] }

  metroConfig?: MetroConfig
  filterModules?: Set<string>
  filterModulesVersion?: Set<string>
  extUuid?: string
  projectConfig?: {
    cacheVersion?: number
  }
}


export type CompileOutput = {
  entryAbsPath: string
  bundleAbsPath: string
  bundleMd5: string
  platform: Platform
}

export default async function compile({
  projectRoot,
  filterModules,
  filterModulesVersion,
  imageBaseURL,

  SDKAlias,

  metroConfig = {},
  bundleConfigs,
  extUuid,
  projectConfig,
}: Options): Promise<CompileOutput[]> {
  dbg('compile start')
  const allStart = Date.now()

  // for its side effect
  let tStart = Date.now()
  validateAnimatedValue(projectRoot, extUuid as string)
  dbg('validateAnimatedValue: %s', Date.now() - tStart)


  tStart = Date.now()
  const defaultConfig = getDefaultConfig({
    projectRoot,
    imageBaseURL,
    filterModules: filterModules || new Set<string>(),
    filterModulesVersion: filterModulesVersion || new Set<string>(),
    SDKAlias,
    mode: 'production',
    extUuid,
    projectConfig,
  })

  const babelTransformerPath = generateTransformerFile({
    projectRoot: projectRoot,
    babelPlugins: metroConfig?.transformer?.babelPlugins,
    hyextTransformerConfig: metroConfig?.transformer?.hyextTransformerConfig,
  })

  // const finalConfig = mergeConfig(defaultConfig, metroConfig)
  const finalConfig = mergeConfig(defaultConfig, {
    ...metroConfig,
    transformer: {
      babelTransformerPath,
    },
  })
  dbg('getConfig: %s', Date.now() - tStart)


  tStart = Date.now()
  const server = new Server({
    ...finalConfig,
    extUuid,
    // athana 容器内无法获取正确的cpu核心数
    maxWorkers: process.env.MAX_WORKERS
      ? Number(process.env.MAX_WORKERS)
      : finalConfig.maxWorkers,
  }, {
    watch: false,
  })
  dbg('new Server: %d', Date.now() - tStart)


  const bundlePlatform = async ({
    entryAbsPath,
    outputDir,
    assetsDir,
    getBundleName,
  }: BundleConfig, platform: Platform): Promise<CompileOutput> => {
    const bundleName = getBundleName ? getBundleName(platform, entryAbsPath) : `${path.basename(entryAbsPath, '.js')}.${platform}.jsbundle`
    const bundleAbsPath = path.resolve(outputDir, bundleName)

    const requestOpts = {
      entryFile: entryAbsPath,
      sourceMapUrl: `${path.basename(bundleAbsPath)}.map`,
      dev: false,
      minify: true,
      platform,
    }

    let localStart = Date.now()
    const bundle = await outputBundle.build(server, requestOpts)
    dbg(`build %s Bundle: %d`, platform, Date.now() - localStart)

    const outputOpts = {
      bundleOutput: bundleAbsPath,
      bundleEncoding: 'utf8',
      dev: false,
      indexedRamBundle: false,
      platform,
      assetsDest: assetsDir || outputDir,
      sourcemapOutput: `${bundleAbsPath}.map`,
      // sourcemapUseAbsolutePath: false,
      // sourcemapSourcesRoot: '',
    }
    localStart = Date.now()
    await outputBundle.save(bundle, outputOpts, logger.info)
    dbg(`save %s Bundle: %d`, platform, Date.now() - localStart)

    if (platform === 'ios') {
      localStart = Date.now()
      const outputAssets = await server.getAssets({
        ...Server.DEFAULT_BUNDLE_OPTIONS,
        ...requestOpts,
        bundleType: 'todo',
      })
      // 只需要保存一份资源即可
      await saveAssets(outputAssets, platform, assetsDir || outputDir)
      dbg('saveAssets %d', Date.now() - localStart)
    }

    const bundleMd5 = await md5(bundleAbsPath)

    return {
      entryAbsPath,
      bundleAbsPath,
      bundleMd5,
      platform,
    }
  }


  const promises: Array<Promise<CompileOutput>> = []
  bundleConfigs.forEach(bundleConfig => {
    (bundleConfig.platforms || ['ios', 'android']).forEach(platform => {
      promises.push(bundlePlatform(bundleConfig, platform))
    })
  })

  try {
    const compileOutputs = await Promise.all(promises)
    return compileOutputs
  } finally {
    server.end()
    if (process.env.MATRIX_BUILD_ID) {
      axios.put('http://localhost:3002/telemetry/statExtra', {
        buildId: process.env.MATRIX_BUILD_ID,
        extra: [[bundleConfigs[0].entryAbsPath.replace(projectRoot, ''), Date.now() - allStart]]
      }).catch(() => 'ignore error')
    }
    dbg('totalCompileTime: %d', Date.now() - allStart)
  }
}
